<?php

    require_once "/var/www/ebook/inc/HTTP/WebDAV/Server.php";
//    require_once "System.php";
    
    /**
     * Filesystem access using WebDAV
     *
     * @access public
     */
    class HTTP_WebDAV_Server_Filesystem extends HTTP_WebDAV_Server 
    {
        /**
         * Root directory for WebDAV access
         *
         * Defaults to webserver document root (set by ServeRequest)
         *
         * @access private
         * @var    string
         */
        var $base = "";

        /** 
         * MySQL Host where property and locking information is stored
         *
         * @access private
         * @var    string
         */
        var $db_host = "";

        /**
         * MySQL database for property/locking information storage
         *
         * @access private
         * @var    string
         */
        var $db_name = "";

        /**
         * MySQL user for property/locking db access
         *
         * @access private
         * @var    string
         */
        var $db_user = "";

        /**
         * MySQL password for property/locking db access
         *
         * @access private
         * @var    string
         */
        var $db_passwd = "";


	/** 
	 * The operating system of the WebDAV client.
	 * This is 'windows', 'unix' or 'unknown'.
	 * (Mac OS X considered as 'unix'.).
	 */
	private $clientOS = 'unknown';

	/**
	 * The flavor of the operating system of the WebDAV client.
	 * This is 'xp', 'osx', or 'unknown'.
	 */
	private $clientOSFlavor = 'unknown';

	/**
	 * The name of some well known browsers, that need special support.
	 * This is either "konqueror", or unknown.
	 */
	private $clientBrowser = 'unknown';


	public function HTTP_WebDAV_Server_Filesystem()
	{

		/** YO **/
		$this->HTTP_WebDAV_Server();
		// Guess operating system, operating system flavor and browser of the webdav client
		//
		// - We need to know the operating system in order to properly
		// hide hidden resources in directory listings.
		//
		// - We need the operating system flavor and the browser to
		// properly support mounting of a webdav folder.
		//
		$userAgent = strtolower($_SERVER['HTTP_USER_AGENT']);

		if (strpos($userAgent,'windows') !== false
		|| strpos($userAgent,'microsoft') !== false)
		{
			$this->clientOS = 'windows';
			$this->clientOSFlavor = 'xp';

		} else if (strpos($userAgent,'darwin') !== false
                || strpos($userAgent,'macintosh') !== false
		|| strpos($userAgent,'linux') !== false
		|| strpos($userAgent,'solaris') !== false
		|| strpos($userAgent,'aix') !== false
		|| strpos($userAgent,'unix') !== false
		|| strpos($userAgent,'gvfs') !== false // nautilus browser uses this ID
		)
		{
			$this->clientOS = 'unix';
			if (strpos($userAgent,'linux') !== false)
			{
				$this->clientOSFlavor = 'linux';
			}
			else if (strpos($userAgent,'macintosh') !== false)
			{
				$this->clientOSFlavor = 'osx';
			}
		}
		if (strpos($userAgent,'konqueror') !== false)
		{
			$this->clientBrowser = 'konqueror';
		}

	}

        /**
         * Serve a webdav request
         *
         * @access public
         * @param  string  
         */
        function ServeRequest($base = false) 
        {
            // special treatment for litmus compliance test
            // reply on its identifier header
            // not needed for the test itself but eases debugging
            if (function_exists("apache_request_headers")) {
				foreach(apache_request_headers() as $key => $value) {
					if (stristr($key,"litmus")) {
						error_log("Litmus test $value");
						header("X-Litmus-reply: ".$value);
					}
				}
			}

            // set root directory, defaults to webserver document root if not set
            if ($base) { 
                $this->base = realpath($base); // TODO throw if not a directory
            } else if (!$this->base) {
                $this->base = $_SERVER['DOCUMENT_ROOT'];
            }
                
            // establish connection to property/locking db
            mysql_connect($this->db_host, $this->db_user, $this->db_passwd) or die(mysql_error());
            mysql_select_db($this->db_name) or die(mysql_error());
            // TODO throw on connection problems

            // let the base class do all the work
            parent::ServeRequest();

        }

        /**
         * No authentication is needed here
         *
         * @access private
         * @param  string  HTTP Authentication type (Basic, Digest, ...)
         * @param  string  Username
         * @param  string  Password
         * @return bool    true on successful authentication
         */
        function check_auth($type, $user, $pass) 
        {
            return true;
        }
	
		function _urlencode($url) 
	    {
        	return strtr($url, array(" "=>"%20",
        	                         "&"=>"%26",
        	                         "<"=>"%3C",
        	                         ">"=>"%3E",
									 "á"=>"%C3%A1",
									 "ó"=>"%C3%B3",
									 "É"=>"%C3%89",
									 "í"=>"%C3%AD",
									 "("=>"%28",
									 ")"=>"%29",
									 ":"=>"%3A",
									 "é"=>"%C3%A9",
									 "ü"=>"%C3%BC",
									 "ñ"=>"%C3%B1",																								
									 ","=>"%2C",												
									 "ú"=>"%C3%BA",												
        	                         ));
		}

        /**
         * getPROPFIND method handler
         *
         * @param  array  general parameter passing array
         * @param  array  return array for file properties
         * @return bool   true on success
         */
        function PROPFIND(&$options, &$files) 
        {


	    $options["path"]=$this->_urlencode($options['path']);
            // gget absolute fs path to requested resource
            $fspath = $this->base . $options["path"];
            
			//compruebo si la ruta es permitida, es decir que contenga la referencia de un curso del cual
			
			//el usuario esta matriculado. Asi evito acceso indebidos

			//primero compruebo que en la url haya una carpeta del tipo ref_xxx (que es un curso)
			if (substr_count($fspath,"ref_") != 0)
			{
				//obtengo el array de cursos de la sesion
				session_start(); 
				$cursos = $_SESSION["courses"];

				$permiso = 0;

				//recorro los cursos
				foreach($cursos as $value) 
				{					
					if (substr_count($fspath,"ref_".$value) != 0) 
	   					$permiso = 1;
				}				
				// o no tiene permiso para acceder...
			    	if (!$permiso) 
				{
					//echo "El usuario ".$_SESSION['userLogin']." no tiene permiso para acceder al recurso.";
					return false;
				}
			} 

            // sanity check
            if (!file_exists($fspath)) {
                return false;
            }

            // prepare property array
            $files["files"] = array();

            // store information for the requested path itself
            $files["files"][] = $this->fileinfo($options["path"]);

            // information for contained resources requested?
            if (!empty($options["depth"]))  { // TODO check for is_dir() first?
                
                // make sure path ends with '/'
                $options["path"] = $this->_slashify($options["path"]);

                // try to open directory

                $handle = @opendir($fspath);
                if ($handle) 
		{
                 	// ok, now get all its contents
			//ahora busco en la base de datos los cursos del alumno.
			//creo una instancia de la calse DB que es la que comunica con la base de datos
			//incluyo la clase que maneja la base de datos
			//include_once("./DB.php");
					
			require_once "/var/www/ebook/inc/HTTP/WebDAV/Server/DB.php";
			$db = new DB("localhost","nombreBD","usuarioBD","passwordBD");		     		     

		     
			$db->connect();     
		
					
                    	while ($filename = readdir($handle)) 
			{			
				//ahora comprebo que el curso que lista, tiene permiso el usuario.
				session_start(); 

				$permiso = $db->checkPermission($_SESSION["userLogin"],$filename);


	                        if ($filename != "." && $filename != ".." && $permiso) 
				{
	                        	$files["files"][] = $this->fileinfo($options["path"].$filename);					
	                        }
        	       }
                    	// TODO recursion needed if "Depth: infinite"
                }
            }

            // ok, all done
            return true;
        } 
        
        /**
         * Get properties for a single file/resource
         *
         * @param  string  resource path
         * @return array   resource properties
         */
        function fileinfo($path) 
        {
            // map URI path to filesystem path
            $fspath = $this->base . $path;

            // create result array
            $info = array();
            // TODO remove slash append code when base clase is able to do it itself
            $info["path"]  = is_dir($fspath) ? $this->_slashify($path) : $path; 
            $info["props"] = array();
            
            // no special beautified displayname here ...
            $info["props"][] = $this->mkprop("displayname", strtoupper($path));
            
            // creation and modification time
            $info["props"][] = $this->mkprop("creationdate",    filectime($fspath));
            $info["props"][] = $this->mkprop("getlastmodified", filemtime($fspath));

            // type and size (caller already made sure that path exists)
            if (is_dir($fspath)) {
                // directory (WebDAV collection)
                $info["props"][] = $this->mkprop("resourcetype", "collection");
                $info["props"][] = $this->mkprop("getcontenttype", "httpd/unix-directory");             
            } else {
                // plain file (WebDAV resource)
                $info["props"][] = $this->mkprop("resourcetype", "");
                if (is_readable($fspath)) {
                    $info["props"][] = $this->mkprop("getcontenttype", $this->_mimetype($fspath));
                } else {
                    $info["props"][] = $this->mkprop("getcontenttype", "application/x-non-readable");
                }               
                $info["props"][] = $this->mkprop("getcontentlength", filesize($fspath));
            }

            // get additional properties from database
            $query = "SELECT ns, name, value FROM properties WHERE path = '$path'";
            $res = mysql_query($query);
            while ($row = mysql_fetch_assoc($res)) {
                $info["props"][] = $this->mkprop($row["ns"], $row["name"], $row["value"]);
            }
            mysql_free_result($res);

            return $info;
        }

        /**
         * detect if a given program is found in the search PATH
         *
         * helper function used by _mimetype() to detect if the 
         * external 'file' utility is available
         *
         * @param  string  program name
         * @param  string  optional search path, defaults to $PATH
         * @return bool    true if executable program found in path
         */
        function _can_execute($name, $path = false) 
        {
            // path defaults to PATH from environment if not set
            if ($path === false) {
                $path = getenv("PATH");
            }
            
            // check method depends on operating system
            if (!strncmp(PHP_OS, "WIN", 3)) {
                // on Windows an appropriate COM or EXE file needs to exist
                $exts = array(".exe", ".com");
                $check_fn = "file_exists";
            } else { 
                // anywhere else we look for an executable file of that name
                $exts = array("");
                $check_fn = "is_executable";
            }
            
            // now check the directories in the path for the program
            foreach (explode(PATH_SEPARATOR, $path) as $dir) {
                // skip invalid path entries
                if (!file_exists($dir)) continue;
                if (!is_dir($dir)) continue;

                // and now look for the file
                foreach ($exts as $ext) {
                    if ($check_fn("$dir/$name".$ext)) return true;
                }
            }

            return false;
        }

        
        /**
         * try to detect the mime type of a file
         *
         * @param  string  file path
         * @return string  guessed mime type
         */
        function _mimetype($fspath) 
        {
            if (@is_dir($fspath)) {
                // directories are easy
                return "httpd/unix-directory"; 
            } else if (function_exists("mime_content_type")) {
                // use mime magic extension if available
                $mime_type = mime_content_type($fspath);
            } else if ($this->_can_execute("file")) {
                // it looks like we have a 'file' command, 
                // lets see it it does have mime support
                $fp = popen("file -i '$fspath' 2>/dev/null", "r");
                $reply = fgets($fp);
                pclose($fp);
                
                // popen will not return an error if the binary was not found
                // and find may not have mime support using "-i"
                // so we test the format of the returned string 
                
                // the reply begins with the requested filename
                if (!strncmp($reply, "$fspath: ", strlen($fspath)+2)) {                     
                    $reply = substr($reply, strlen($fspath)+2);
                    // followed by the mime type (maybe including options)
                    if (preg_match('/^[[:alnum:]_-]+/[[:alnum:]_-]+;?.*/', $reply, $matches)) {
                        $mime_type = $matches[0];
                    }
                }
            } 
            
            if (empty($mime_type)) {
                // Fallback solution: try to guess the type by the file extension
                // TODO: add more ...
                // TODO: it has been suggested to delegate mimetype detection 
                //       to apache but this has at least three issues:
                //       - works only with apache
                //       - needs file to be within the document tree
                //       - requires apache mod_magic 
                // TODO: can we use the registry for this on Windows?
                //       OTOH if the server is Windos the clients are likely to 
                //       be Windows, too, and tend do ignore the Content-Type
                //       anyway (overriding it with information taken from
                //       the registry)
                // TODO: have a seperate PEAR class for mimetype detection?
                switch (strtolower(strrchr(basename($fspath), "."))) {
                case ".html":
                    $mime_type = "text/html";
                    break;
                case ".gif":
                    $mime_type = "image/gif";
                    break;
                case ".jpg":
                    $mime_type = "image/jpeg";
                    break;
                default: 
                    $mime_type = "application/octet-stream";
                    break;
                }
            }
            
            return $mime_type;
        }

        /**
         * GET method handler
         * 
         * @param  array  parameter passing array
         * @return bool   true on success
         */
        function GET(&$options) 
        {
        	// get absolute fs path to requested resource
	        $fspath = $this->base . $options["path"];

		//compruebo si la ruta es permitida, es decir que contenga la referencia de un curso del cual
			
		//el usuario esta matriculado. Asi evito acceso indebidos
		//primero compruebo que en la url haya una carpeta del tipo ref_xxx (que es un curso)
		if (substr_count($fspath,"ref_") != 0)
		{
			//obtengo el array de cursos de la sesion
			session_start(); 
			$cursos = $_SESSION["courses"];

			$permiso = 0;

			//recorro los cursos
			foreach($cursos as $value) 
			{
				if (substr_count($fspath,"ref_".$value) != 0) 
   					$permiso = 1;
			}			
			// o no tiene permiso para acceder...
	    	    	if (!$permiso) 
			{
				echo "El usuario ".$_SESSION['userLogin']." no tiene permiso para acceder al recurso.";
				return false;
			}
		} 
				
	        // sanity check
		//si el fichero no existe
        	if (!file_exists($fspath)) return false;
            
	        // is this a collection?
        	if (is_dir($fspath)) {
                return $this->GetDir($fspath, $options);
	        }
            
        	    // detect resource type
        	    $options['mimetype'] = $this->_mimetype($fspath); 
                
        	    // detect modification time
	       	    // see rfc2518, section 13.7
        	    // some clients seem to treat this as a reverse rule
        	    // requiering a Last-Modified header if the getlastmodified header was set
	            $options['mtime'] = filemtime($fspath);
            
	            // detect resource size
	            $options['size'] = filesize($fspath);
            
	            // no need to check result here, it is handled by the base class
	            $options['stream'] = fopen($fspath, "r");
            
	            return true;
        }

        /**
         * GET method handler for directories
         *
         * This is a very simple mod_index lookalike.
         * See RFC 2518, Section 8.4 on GET/HEAD for collections
         *
         * @param  string  directory path
         * @return void    function has to handle HTTP response itself
         */
        function GetDir($fspath, &$options) 
        {
		//conecto con la base de datos.
		require_once "/var/www/ebook/inc/HTTP/WebDAV/Server/DB.php";
		$db = new DB("localhost","nombreBD","usuarioBD","passwordBD");		     		     

		$db->connect();     
					
		$path = $this->_slashify($options["path"]);
	        if ($path != $options["path"]) 
		{
	                header("Location: ".$this->base_uri.$path);
	                exit;
        	}

	        // fixed width directory column format
        	$format = "%15s  %-19s  %-s\n";

	        $handle = @opendir($fspath);
	      	if (!$handle) 
		{
                	return false;
	        }

		//inicio la sesion
		session_start(); 


		//si esta actulizando el sistema, se refresca cada 5 segundos. Sino cada 300 segundos.

		if ($db->checkIfSomeCourseIsUpdating())	
			$refresco = 5;
		else
			$refresco = 300;


	        echo "<html><head><meta http-equiv='refresh' content='$refresco';content='text/html; charset=UTF-8' http-equiv='content-type'><title>Usuario: ".$_SESSION["userLogin"]." Index of ".urldecode($options['path'])."</title></head>\n";
echo "<font size=4>";
		echo "<center><h2>eBookSync</h2></center>";            
	        echo "Identificado como: <b>".$_SESSION["userLogin"]."</b></br>";

		//miro si en el path hay alguna ruta del tipo /ref_xxx, si la hay, la formateo
		if (substr_count($options['path'],"ref") != 0)
		{
			//divido la ruta en partes por la /
			$pieces = explode("/", $options['path']);
			//miro si en cada parte hay algo del tipo ref_xxx, busco su nombre y lo sustituyo
			foreach($pieces as $value)
			{
				//busco la parte que tiene ref_xxx
				if (substr_count($value,"ref") != 0)
				{		
					//busco en la base de datos el nombre del curso y lo sustituyo en el path
					$options['path'] = str_replace($value,utf8_encode($db->getNameOfCourse($value)),$options['path']);					

				}
			}
		}

		echo "Carpeta actual de navegaci&oacute;n:<b> ".urldecode($options['path'])."</b></br>";

	        echo "<pre>";

	        echo "<hr>";
			
		echo "<center>";
		
		echo "<a href=\"javascript:history.back()\" class=\"enlace-volver\" title=\"Volver\">Volver</a>   ";
		echo "<a href=".$_SERVER["PHP_SELF"].">Actualizar</a> ";
		echo "</center>";
		echo "<table border='0' align='center' CELLSPACING='10'>";
		echo "<tr>";
		echo "<td align='center'><i><b>Recurso</b></i></td>";
		echo "<td align='center'><i><b>Fecha de ultima modificaci&oacute;n</b></i></td>";
		echo "</tr>";

	        while ($filename = readdir($handle)) 	
		{
			//variables de estado
			//curso --> para saber si el recurso acutal es un curso
			//updating --> en caso de ser un curso, informa de si esta siendo actualizado o no
			$curso = 0;
			$updating = 0;
	
			//ahora comprebo que el curso que lista, tiene permiso el usuario.
			$permiso = $db->checkPermission($_SESSION["userLogin"],$filename);
	
        	        if ($filename != "." && $filename != ".." && $permiso) 
			{
        	            $fullpath = $fspath."/".$filename;
        	            $name = htmlspecialchars($filename);
		
			    //compruebo si name es un curso (lo se porque tendre un ref_xxx)

			   //compruebo si el recurso actual es un directorio para diferenciarlo de los ficheros.
  			   list($rutaPath, $restoRuta) = split("/var/www/recursos", $fspath);
			   $dir = $db->checkIfResourceIsDirectory($restoRuta.$filename);
			
			   if (substr_count($name,"ref") != 0)
			   {
					//busco cual es su nombre
					$nameFormat = utf8_encode($db->getNameOfCourse($name));
					$updating = $db->getIfUpdatingCourse($name);
					$curso = 1;
			   }
			   //como no es un curso, lo muestro tal y como estaba. Para ello, el nombre formateado es el mismo que el nombre sin formatear
			   else
			   {
					$nameFormat = $name;
					list($rutaPath, $restoRuta) = split("/var/www/recursos", $fspath);
					$updating = $db->getIfUpdatingResource($restoRuta.$filename);
  			   }

		echo "<tr>";
	
		//si el recurso actual es un directorio,le pongo unos corchetes para diferenciarlo
		if ($dir == 0)
			echo "<td align='left' style='font-size:16'><a href='$this->base_uri$path$name'>".urldecode($nameFormat)."</a></td>";
		else
			echo "<td align='left' style='font-size:16'><a href='$this->base_uri$path$name'>[[ ".urldecode($nameFormat)." ]]</a></td>";

		echo "<td align='center' style='font-size:16'>".strftime("%H:%M:%S %d-%m-%Y", filemtime($fullpath))."</td>";
		if ($updating == 1)
			echo "<td align='center' style='font-size:16'>Actualizando...</td>";
		else
			echo "<td align='center' style='font-size:16'>-</td>";
		echo "</tr>";

            	}
            }

            echo "</pre>";
echo "</font>";
            closedir($handle);

			
			
	    echo "</table>";
	    echo "<center>";

	    echo "<a href=\"javascript:history.back()\" class=\"enlace-volver\" title=\"Volver\">Volver</a>   ";
	    echo "<a href=".$_SERVER["PHP_SELF"].">Actualizar</a> ";
 	    echo "</center>";

            echo "</html>\n";

            exit;
        }

        /**
         * PUT method handler
         * 
         * @param  array  parameter passing array
         * @return bool   true on success
         */
        function PUT(&$options) 
        {
            $fspath = $this->base . $options["path"];

            if (!@is_dir(dirname($fspath))) {
                return "409 Conflict";
            }

            $options["new"] = ! file_exists($fspath);

            $fp = fopen($fspath, "w");

            return $fp;
        }


        /**
         * MKCOL method handler
         *
         * @param  array  general parameter passing array
         * @return bool   true on success
         */
        function MKCOL($options) 
        {           
            $path = $this->base .$options["path"];
            $parent = dirname($path);
            $name = basename($path);

            if (!file_exists($parent)) {
                return "409 Conflict";
            }

            if (!is_dir($parent)) {
                return "403 Forbidden";
            }

            if ( file_exists($parent."/".$name) ) {
                return "405 Method not allowed";
            }

            if (!empty($_SERVER["CONTENT_LENGTH"])) { // no body parsing yet
                return "415 Unsupported media type";
            }
            
            $stat = mkdir ($parent."/".$name,0777);
            if (!$stat) {
                return "403 Forbidden";                 
            }

            return ("201 Created");
        }
        
        
        /**
         * DELETE method handler
         *
         * @param  array  general parameter passing array
         * @return bool   true on success
         */
        function DELETE($options) 
        {
            $path = $this->base . "/" .$options["path"];

            if (!file_exists($path)) {
                return "404 Not found";
            }

            if (is_dir($path)) {
                $query = "DELETE FROM properties WHERE path LIKE '".$this->_slashify($options["path"])."%'";
                mysql_query($query);
                System::rm("-rf $path");
            } else {
                unlink ($path);
            }
            $query = "DELETE FROM properties WHERE path = '$options[path]'";
            mysql_query($query);

            return "204 No Content";
        }


        /**
         * MOVE method handler
         *
         * @param  array  general parameter passing array
         * @return bool   true on success
         */
        function MOVE($options) 
        {
            return $this->COPY($options, true);
        }

        /**
         * COPY method handler
         *
         * @param  array  general parameter passing array
         * @return bool   true on success
         */
        function COPY($options, $del=false) 
        {
            // TODO Property updates still broken (Litmus should detect this?)

            if (!empty($_SERVER["CONTENT_LENGTH"])) { // no body parsing yet
                return "415 Unsupported media type";
            }

            // no copying to different WebDAV Servers yet
            if (isset($options["dest_url"])) {
                return "502 bad gateway";
            }

            $source = $this->base .$options["path"];
            if (!file_exists($source)) return "404 Not found";

            $dest = $this->base . $options["dest"];

            $new = !file_exists($dest);
            $existing_col = false;

            if (!$new) {
                if ($del && is_dir($dest)) {
                    if (!$options["overwrite"]) {
                        return "412 precondition failed";
                    }
                    $dest .= basename($source);
                    if (file_exists($dest)) {
                        $options["dest"] .= basename($source);
                    } else {
                        $new = true;
                        $existing_col = true;
                    }
                }
            }

            if (!$new) {
                if ($options["overwrite"]) {
                    $stat = $this->DELETE(array("path" => $options["dest"]));
                    if (($stat{0} != "2") && (substr($stat, 0, 3) != "404")) {
                        return $stat; 
                    }
                } else {                
                    return "412 precondition failed";
                }
            }

            if (is_dir($source) && ($options["depth"] != "infinity")) {
                // RFC 2518 Section 9.2, last paragraph
                return "400 Bad request";
            }

            if ($del) {
                if (!rename($source, $dest)) {
                    return "500 Internal server error";
                }
                $destpath = $this->_unslashify($options["dest"]);
                if (is_dir($source)) {
                    $query = "UPDATE properties 
                                 SET path = REPLACE(path, '".$options["path"]."', '".$destpath."') 
                               WHERE path LIKE '".$this->_slashify($options["path"])."%'";
                    mysql_query($query);
                }

                $query = "UPDATE properties 
                             SET path = '".$destpath."'
                           WHERE path = '".$options["path"]."'";
                mysql_query($query);
            } else {
                if (is_dir($source)) {
                    $files = System::find($source);
                    $files = array_reverse($files);
                } else {
                    $files = array($source);
                }

                if (!is_array($files) || empty($files)) {
                    return "500 Internal server error";
                }
                    
                
                foreach ($files as $file) {
                    if (is_dir($file)) {
                      $file = $this->_slashify($file);
                    }

                    $destfile = str_replace($source, $dest, $file);
                    
                    if (is_dir($file)) {
                        if (!is_dir($destfile)) {
                            // TODO "mkdir -p" here? (only natively supported by PHP 5) 
                            if (!mkdir($destfile)) {
                                return "409 Conflict";
                            }
                        } else {
                          error_log("existing dir '$destfile'");
                        }
                    } else {
                        if (!copy($file, $destfile)) {
                            return "409 Conflict";
                        }
                    }
                }

                $query = "INSERT INTO properties SELECT ... FROM properties WHERE path = '".$options['path']."'";
            }

            return ($new && !$existing_col) ? "201 Created" : "204 No Content";         
        }

        /**
         * PROPPATCH method handler
         *
         * @param  array  general parameter passing array
         * @return bool   true on success
         */
        function PROPPATCH(&$options) 
        {
            global $prefs, $tab;

            $msg = "";
            
            $path = $options["path"];
            
            $dir = dirname($path)."/";
            $base = basename($path);
            
            foreach($options["props"] as $key => $prop) {
                if ($prop["ns"] == "DAV:") {
                    $options["props"][$key]['status'] = "403 Forbidden";
                } else {
                    if (isset($prop["val"])) {
                        $query = "REPLACE INTO properties SET path = '$options[path]', name = '$prop[name]', ns= '$prop[ns]', value = '$prop[val]'";
                        error_log($query);
                    } else {
                        $query = "DELETE FROM properties WHERE path = '$options[path]' AND name = '$prop[name]' AND ns = '$prop[ns]'";
                    }       
                    mysql_query($query);
                }
            }
                        
            return "";
        }


        /**
         * LOCK method handler
         *
         * @param  array  general parameter passing array
         * @return bool   true on success
         */
        function LOCK(&$options) 
        {
            if (isset($options["update"])) { // Lock Update
                $query = "UPDATE locks SET expires = ".(time()+300);
                mysql_query($query);
                
                if (mysql_affected_rows()) {
                    $options["timeout"] = 300; // 5min hardcoded
                    return true;
                } else {
                    return false;
                }
            }
            
            $options["timeout"] = time()+300; // 5min. hardcoded

            $query = "INSERT INTO locks
                        SET token   = '$options[locktoken]'
                          , path    = '$options[path]'
                          , owner   = '$options[owner]'
                          , expires = '$options[timeout]'
                          , exclusivelock  = " .($options['scope'] === "exclusive" ? "1" : "0")
                ;
            mysql_query($query);

            return mysql_affected_rows() ? "200 OK" : "409 Conflict";
        }

        /**
         * UNLOCK method handler
         *
         * @param  array  general parameter passing array
         * @return bool   true on success
         */
        function UNLOCK(&$options) 
        {
            $query = "DELETE FROM locks
                      WHERE path = '$options[path]'
                        AND token = '$options[token]'";
            mysql_query($query);

            return mysql_affected_rows() ? "204 No Content" : "409 Conflict";
        }

        /**
         * checkLock() helper
         *
         * @param  string resource path to check for locks
         * @return bool   true on success
         */
        function checkLock($path) 
        {
            $result = false;
            
            $query = "SELECT owner, token, expires, exclusivelock
                  FROM locks
                 WHERE path = '$path'
               ";
            $res = mysql_query($query);

            if ($res) {
                $row = mysql_fetch_array($res);
                mysql_free_result($res);

                if ($row) {
                    $result = array( "type"    => "write",
                                                     "scope"   => $row["exclusivelock"] ? "exclusive" : "shared",
                                                     "depth"   => 0,
                                                     "owner"   => $row['owner'],
                                                     "token"   => $row['token'],
                                                     "expires" => $row['expires']
                                                     );
                }
            }

            return $result;
        }


        /**
         * create database tables for property and lock storage
         *
         * @param  void
         * @return bool   true on success
         */
        function create_database() 
        {
            // TODO
            return false;
        }
        
	/**
	* Writes a message to the logfile.,
	*
	* @param  message String.
	* @return void.
	*/
	protected function writelog($message)
	{
		$fh = fopen('/var/www/ebook/log/webdav.log', 'a');
		fwrite($fh, date('Y-m-d H:i:s '));
		fwrite($fh, $message);
		fwrite($fh, "\n");
		fclose($fh);
	}

    }


?>
